##################################################################
##  
##  Aseryla project [http://aseryla.16mb.com]
##  Information Extraction task
##
##
 
 
import sys      #
import time     # 
import socket   # aseryla connections

import termios
 # to catch keyboard interruptions
from select import select
 

aserPort = 9991 # assumes the aseryla server is running whit this "non-standard" ports and autosave = 0(disabled)
nlpPort = 9992 
 
 
 
 
##  UTILS AND AUXILIARY COMMON FUNCTIONS ####################################################################################################################
 
def appendFile(content, filename):
    f = open(filename, 'a')
    f.write(content)
    f.close()
 
# log formatter
def log_entry(article, snts, time):
    appendFile(article.replace(',', ';') + ", " + str(snts) + ", " + time + "\n", "wikiprocess.log")
     
 
 
# convert milliseconds time to natural format in seconds
def trans_time(start):
#    return "%6.3f" % (float(int(round((time.clock() - start) * 1000))) / 1000)  windows
     return str(int(time.time() - start))


# capture keyboard
def kbhit():

    dr,dw,de = select([sys.stdin], [], [], 0)

    return dr <> []

    

    
## -------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
  
 
 
 
 
 
 
## Aseryla server communication ###################################################################################################################
 
EndCommSymbol = "\b"
 
# manages a socket read message
def socket_receive(sock):
    total_data=[]
    data=''
    while True:
            data=sock.recv(8192)
            if EndCommSymbol in data:
                total_data.append(data[:data.find(EndCommSymbol)])
                break
            total_data.append(data)
            if len(total_data)>1:
                #check if end_of_data was split
                last_pair=total_data[-2]+total_data[-1]
                if EndCommSymbol in last_pair:
                    total_data[-2]=last_pair[:last_pair.find(EndCommSymbol)]
                    total_data.pop()
                    break
    return ''.join(total_data)
	
 
	
 
 
# send a message to the Aseryla server, and reads the response
def send_process(order, parameter, port):
     
    try:
        # connect with the server
        sock = socket.socket()
        sock.connect(("localhost", port))
 
        #send the entire message with the \b end symbol at the end (the expected final symbol in the server)
        sock.sendall(" " + order + " " + parameter + EndCommSymbol)
 
        # ok = the results list 
        # K0 = no content
        # K1 = no nlp
        # K2 = error process
        # K3 = no server
 
        # read the response
        res = socket_receive(sock)
 
        # close the communication    
        sock.close()
 
        return res[1:-1]
    except:
        return "K3"
 
 
 
 
 
# given a text, get the cleaned and parsed sentences
def split_sentences(text):
    res = send_process("Tp", text, aserPort)
  
    if res == '-': # the results are higher than 8kb
        return "K2"
 
    if res == '/':
        return "K0"
 
    return res



   
# process a bunch of sentences
def process_text(text):

## Error: Lexical parsing
## Error: Empty content
## Error: NLP kit not available
## Error: processing grammar on ("sentence")
## Error: Semantical Failure on ("sentence")
## Error: Undefined during the language processing
## number of processed sentence
    return send_process("Tb 0 ", text, aserPort)
 
 
# 
def stop_aseryla():
    send_process("@x", " ", aserPort)

def stop_nlp():
    send_process("#exit#", " ", nlpPort)

     
# checks if the Aseryla server is running properly
def check_aser_server():
    return send_process("@h", "", aserPort) != 'K3'
  

# checks if the nlp server is running properly
def check_nlp_server():
    res = send_process("@n", "", aserPort)
    
    return res != "K3" and res[0] != '0'
  


 
## -----------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
####################################################################################################################
##  MAIN  ##########################################################################################################
####################################################################################################################
 
  
 
# surf the web and process the extracted sentences 
def process():

    if not check_aser_server():
        print "NO SERVER"
        return 

    if not check_nlp_server():
        print "NO NLP"
        return


    send_process("@w", "0", aserPort) # disable autosave mechanism
    

    fr = open("last.txt", 'rt')
    last = fr.read()
    fr.close()

    processed = 0
    currentPage = ""

    start = time.time()

    with open('wiki1concept.txt', 'rt') as wfile:
        for line in wfile:

            
            idx = line.find(':')
            title= line[0:idx]

            processed = processed + 1
            
            ## skip the already processed articles
            if len(last) > 0:
                if last == title:
                    last = ""
                    print "skip", processed, trans_time(start)
                    processed = 0

                continue

            try:

                start = time.time()

                fs = open("last.txt", 'wt')
                fs.write(title)
                fs.close()



                text = line[idx+1:]
                # analyse and extract the ilc to each sentence ###############################################################
                res = process_text(text)
                    ## Error: Lexical parsing
                    ## Error: NLP kit not available
                    ## Error: processing grammar on ("sentence")
                    ## Error: Semantical Failure on ("sentence")
                    ## Error: Undefined during the language processing
                    ## number of processed sentence (if OK)

                if res == "K3":
                    print "SERVER NOT AVAILABLE (K3)"
                    return 

                if res[0:7] == "Error: ":
                    if res == "Error: Empty content":
                        continue
						
		    if res == "Error: NLP kit not available":
                        print "NLP NOT AVAILABLE"
		    else:
                        appendFile(title + "\n" + text, "failed.txt")

                    return

                                     
                # end of page processing
                tt = trans_time(start)
                log_entry(title, res, tt)
                print processed, title, res, tt

            
                # press escape key to exit
                if kbhit() and ord(sys.stdin.read(1)) == 27:
                    break


                # auto-backup
                if processed % 1000 == 0:
                    print "backup ... "
                    send_process("@v", "0", aserPort) # force aseryla memory saves (no tidy up)
                
            except Exception as e:
                print "ERROR: ", title, str(e), 
                print text
                return


    print "exiting ..."
    send_process("@v", "1", aserPort) # remind this action do not cleanup the memory, it's only tidy up when the system is shutdown, and if no changes no saves
 
  
 
     
 
 
 
 
 
 
 
 
##################################
##################################
##################################
 
 
# to catch keyboar interruptions  (this not work using the Python Idle)
fd = sys.stdin.fileno()

new_term = termios.tcgetattr(fd)

old_term = termios.tcgetattr(fd)

new_term[3] = (new_term[3] & ~termios.ICANON & ~termios.ECHO)
termios.tcsetattr(fd, termios.TCSAFLUSH, new_term) # switch to unbuffered terminal



process()


termios.tcsetattr(fd, termios.TCSAFLUSH, old_term) # switch to normal terminal
